#pragma once

#pragma comment (lib, "legacy_stdio_definitions.lib")

#include <vector>
#include <GL/glut.h>
#include <GL/glaux.h>
using namespace std;
using std::vector;

class Texture
{
public:
	int numTextures;
	int texID;
	vector<AUX_RGBImageRec*> texRecList;

	Texture() 
	{
		numTextures = 0, texID = -1;
	}

	~Texture() 
	{
		for(int i=0; i<numTextures; i++) 
		{
			AUX_RGBImageRec *texRec = texRecList[i];
			if(texRec->data) free(texRec->data);
			free(texRec);
		}
	}

	void Load(const char *filename, int &width, int &height)
	{
		texID = numTextures;
		numTextures++;

		AUX_RGBImageRec *texRec = LoadBMPFile(filename);
		texRecList.push_back(texRec);

		width = texRecList[texID]->sizeX, height = texRecList[texID]->sizeY;

		initTexture();
	}

protected:
	AUX_RGBImageRec* LoadBMPFile(const char* filename)
	{
		FILE* hFile = NULL;
		if (!filename) return NULL;

		if (fopen_s(&hFile, filename, "r") == 0)
		{
			fclose(hFile);

#ifdef UNICODE
			int strSize = MultiByteToWideChar(CP_ACP, 0, filename, -1, NULL, NULL);
			wchar_t* pStr = new WCHAR[strSize];
			MultiByteToWideChar(CP_ACP, 0, filename, strlen(filename) + 1, pStr, strSize);
			return auxDIBImageLoad(pStr);
#else
			return auxDIBImageLoad(filename);
#endif
		}

		return NULL;
	}

	void initTexture()
	{
		glBindTexture(GL_TEXTURE_2D, texID);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, texRecList[texID]->sizeX, texRecList[texID]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, texRecList[texID]->data);
	}

};
